function objs = im2obj5_rp(im, processopt, processparam, thresh, fitstr, ...
    try1pernhood, usenhoodctrs, alldimg)
% im2obj : finds objects in a series of images (e.g. an image stack)
%          by calling fo5_rp.m for each 2D image
%
% Inputs:
% im      : 3D matrix of 2D images 
%           OR structure containing as fields file name information (e.g. 
%              from getnumfilelist, put into a structured variable): 
%              contains fbase, frmin, frmax, formatstr, ext ismultipage
%           OR leave empty to prompt user for file name information 
%           If images are loaded sequentially using file name information,
%           the full data set is NOT stored in memory.
% processopt = option for processing to find neighborhoods -- see fo5_rp.m
% processparam:  Parameters for processing  -- see fo5_rp.m
% thresh : intensity threshold  -- see fo5_rp.m
% fitstr : String that selects the fitting option -- see fo5_rp.m
% try1pernhood : -- see fo5_rp.m
% usenhoodctrs : If true, for frames >= 2, use array of neighborhood centers
%         from previous frame as input to present frame for determining neighborhood center.
%         see fo5_rp.m.  Optional -- default FALSE.  
% alldimg : Optional.  Pre-calculated dilated images (3D array), to avoid 
%        slower frame-by-frame calculations.  Useful only if no filtering is 
%        used for regionfinding (processopt=='none'); otherwise ignored in
%        calcthreshpoints.m
%        Default empty.
%
% Output:
% objs    : object matrix with following form:
%
%             objs = [x;
%                     y; 
%                     mass; (brightness)
%                     particleid; 
%                     frame; 
%                     trackid;
%                     sigma (width)]
%           frame field is set by im2obj().  
%
% Calls fo5_rp.m 
% Modified by Raghuveer Parthasarathy, 16 April, 2007 (original):
% August 6, 2012: Create im2obj5_rp.m, based on earlier im2obj_rp.m
% Last modified: Feb. 28, 2013 (allow input of pre-calc. dilated images)

% Fitting option
if ~exist('fitstr', 'var') || isempty(fitstr)
    disp('Default fitting option: radial-symmetry-based');
    fitstr = 'radial';
end
% Default for using prev. frame information
if ~exist('usenhoodctrs', 'var') || isempty(usenhoodctrs)
    usenhoodctrs = false;
end
if ~exist('try1pernhood', 'var') || isempty(try1pernhood)
    try1pernhood = false;
end
if ~exist('alldimg', 'var') 
    alldimg = [];
end
if ~isempty(alldimg) && ~strcmpi(processopt, 'none')
    disp('im2obj5_rp: warning -- input dilated images will be ignored.')
end

% Is a 3D stack of images input, or file name info, or neither?
if isstruct(im)
    % a structure
    fbase = im.fbase;
    frmin = im.frmin;
    frmax = im.frmax;
    formatstr = im.formatstr;
    ext = im.ext;
    ismultipage = im.ismultipage;
    inputstack = false;
elseif isempty(im)
    % load file name information
    [fbase, frmin, frmax, formatstr, ~, ~, ~, ext, ismultipage] = ...
        getnumfilelist;
    inputstack = false;
else
    % 3D stack of images
    inputstack = true;
end    

% Get nonlinear fitting options, to avoid repeated calls
% These are only used for non-linear Gaussian fitting, but it doesn't hurt
% to define them and pass them on to fo4_rp.m 
lsqoptions = optimset('lsqnonlin');

% Calculate dilated 
objs = [];
% number of frames
if inputstack
    nf = size(im,3); 
else
    nf = frmax - frmin + 1;
end

%% Loop through frames
progtitle = sprintf('im2obj5_{rp}: Finding objects...  '); 
if (nf > 1)
    progbar = waitbar(0, progtitle);  % will display progress
end
for j = 1:nf
    if inputstack
        imj = im(:,:,j);
    else
        if ~ismultipage
            framestr = sprintf(formatstr, j+frmin-1);
            imj  = imread(strcat(fbase, framestr, ext));
        else
            % multipage TIFF
            imj  = imread(strcat(fbase, ext), j);
        end
    end
    if usenhoodctrs && (j>1)
        nhoodctrs = [tmpobj(1,:)' tmpobj(2,:)'];  % use x, y positions from
                   % previous frame for neighborhood centers of this frame
    else
        nhoodctrs = [];
    end
    if isempty(alldimg)
        dimg = [];
    else
        dimg = alldimg(:,:,j);  % pre-calculated dilated image.
    end
    tmpobj = fo5_rp(imj, processopt, processparam, thresh, fitstr, ...
             try1pernhood, nhoodctrs, lsqoptions, dimg);
    tmpobj(5,:) = j;
    objs = [objs tmpobj];
    % show progress
    if mod(j,100)==0
        waitbar(j/nf, progbar, ...
            strcat(progtitle, sprintf('frame %d of %d', j, nf)));
    end
end
if nf>1
    close(progbar)
end

%% -----------------------------------------------------------------------
% Check for likely multiple identifications of the same particle
%   --- find pairs within the same frame that are separated by < "object
%   size"
% If these are found, user may want to re-run the function with a larger
%   value of objsize. (Generally unimportant.)
% Won't display all close pair information -- user can re-examine
% Whatever the processing option, "object size" will be ~ the largest of
% elements 1-2 of processparam
if length(processparam)>1
    objsize = max(processparam(1:2));
else
    objsize = processparam;
end
unqframes = unique(objs(5,:)); % get unique frame numbers
closepairlog = [];
for j=unqframes,
    objframe = objs(:,objs(5,:)==j);  
        % columns of the object matrix for this frame
    if size(objframe,2)>1
        % more than one particle found in this frame
        % consider all particle pairs, find any for which
        % separation < objsize
        allr = [objframe(1,:); objframe(2,:)];  % 2 x "N" matrix of x,y
        % Euclidean distance matrix.
        % uses method of "distance.m" by Roland Bunschoten (MATLAB file exchange)
        aa=sum(allr.*allr,1);
        d = sqrt(abs(aa( ones(size(aa,2),1), :)' + aa( ones(size(aa,2),1), :) - 2*(allr'*allr)));
        if length(objsize)<2
            isclose = (d < objsize);  % close pairs, and d for same pairs
        else
            isclose = (d < objsize(2));  % close pairs, and d for same pairs
        end
        closepair = (sum(isclose(:)) - size(objframe,2))/2.0;  
        templog = [j; closepair];
        closepairlog = [closepairlog templog];  % row 1: frame no; row 2: no. close pairs in this frame
    end
end
if ~isempty(closepairlog)
    if sum(closepairlog(2,:)>0)
        disp('Close pairs found -- recommend re-running with larger objsize.');
        clpairlogfr = closepairlog(:,closepairlog(2,:)>0);
        nclframes = size(clpairlogfr,2);
        medcl = median(clpairlogfr(2,:));
        fs = sprintf('   Number of frames with close pairs: %d out of %d total', nclframes, length(unqframes)); disp(fs);
        fs = sprintf('   In these, median number of close pairs: %d', medcl); disp(fs);
    end
end
