% trackedit.m
% 
% function to "edit" tracks, keeping only those that 
%  -- span a minimal number of frames
%  -- have at least a minimal amount of variance (i.e. aren't stuck)
%
% input
%   objs -- linked object matrix, from nnlink_rp.m  (See that file, or
%           im2obj_rp for object structure)
%   minNframes -- minimal number of frames (optional;
%           leave empty to input during analysis)
%   minstd -- minimal standard deviation of tracks to retain, px (optional;
%           leave empty to input during analysis)
%   dispopt -- 2-element array. 
%           1) Show track information if true (optional; default false)
%           2) Show graph of track lengths if true (default true)
%
% output
%   objs_out -- tracks that are kept (at least minNframes)
% 
% Method:
% identify tracks
% identify max. track length (frames)
% plot tracks -- lines from startframe to endframe
% keep only tracks that span at least minNframes frames; renumber track ids
%
% Raghuveer Parthasarathy
% March 10, 2009
% June 27, 2016: better treatment of default options, plots
% last modified July 10, 2016: 2nd dispopt element, to avoid track figure


function [objs_out] = trackedit(objs, minNframes, minstd, dispopt)

allids = objs(6,:);
utrk = unique(allids);  % all the unique track ids

startframe = min(objs(5,:));  % minimal frame #
endframe = max(objs(5,:));  % maximal frame #

if ~exist('minNframes', 'var') 
    minNframes = []; % user will assign later
end

if ~exist('minstd', 'var') 
    minstd = []; % user will assign later
end

if ~exist('dispopt', 'var') || isempty(dispopt)
    dispopt = [false true];
end
if length(dispopt)==1
    dispopt(2) = true;
end

%% Remove stuck tracks (std. dev. below threshold)

stdtrk = zeros(1,length(utrk));
k=1;
for j = utrk
    % loop through each track
    trtmp = objs(:, allids==j);  % objects that are part of track j
    Nframej = size(trtmp,2);  % length of this track (frames)
    x = trtmp(1,:);  % all x positions
    x = x(~isnan(x));  
    y = trtmp(2,:);  % all y positions
    y = y(~isnan(y));
    stdtrk(k) = sqrt((var(x)+var(y))/Nframej);  
        % standard deviation of position, px per frame
    k = k+1;
end

if isempty(minstd)
    [N, vx] = hist(stdtrk,round(length(utrk)/5));
    figure('position', [100 100 500 500]);
    semilogx(vx, N, 'ko');
    xlabel('standard deviation of position, pixels');
    minstd = input('Enter new standard dev. cutoff (px/frame): ');
end

if minstd>0.0
    objs_out1 = zeros(size(objs));  % the largest it could possibly be
    k=1;  % for re-numbering tracks
    nc = 1;  % number of columns, for re-sizing the array
    progtitle = sprintf('trackedit: Editing tracks...  ');
    progbar = waitbar(0, progtitle);  % will display progress
    for j=1:length(utrk)
        % loop through each track
        if stdtrk(j) >= minstd
            % keep this track
            trtmp = objs(:, allids==utrk(j));  % objects that are part of track j
            trtmp(6,:) = k;  % renumber track id
            objs_out1(:,nc:(nc+size(trtmp,2))-1) = trtmp; % keep these
            k = k+1;
            nc = nc + size(trtmp,2);
        end
        if (mod(j,100)==0)
            waitbar(j/length(utrk), progbar, progtitle);
        end
    end
    close(progbar)
    nc = nc-1;
    fs = sprintf('Keeping %d out of %d tracks', k-1, length(utrk)); disp(fs);
    objs_out1 = objs_out1(:,1:nc);  % "re-sizing" the array
    % Recalculate...
    allids = objs_out1(6,:);
    utrk = unique(allids);  % all the unique track ids
else
    % do nothing
    objs_out1 = objs;
end


%% Graph of track lengths

if dispopt(2)
    % For plotting "lines" of all tracks
    figure('position', [300 200 500 500]);
    axis([startframe endframe 0 max(utrk)]);
    xlabel('Frame');
    ylabel('Track id');
    hold on
    box on
    for j = utrk
        % loop through each track
        trtmp = objs_out1(:, ismember(objs_out1(6,:),j));  % objects that are part of track j
        plot([min(trtmp(5,:)) max(trtmp(5,:))], j*[1 1], '-', 'color', 0.3*mod(j,2)*[1 1 1]);
    end
    title('Track lengths (original)')
end

%% Remove short tracks (user input)

fs = sprintf('Cutoff frame no. = %d', minNframes'); disp(fs);
if isempty(minNframes)
    minNframes = input('Enter new cutoff (minimal number of frames): ');
end
objs_out = zeros(size(objs_out1));  % the largest it could possibly be
k=1;  % for re-numbering tracks
nc = 1;  % number of columns, for re-sizing the array
if dispopt(2)
    figure('position', [600 200 500 500]);
    axis([startframe endframe 0 max(utrk)]);
    xlabel('Frame');
    ylabel('Track id');
    hold on
    box on
end
progtitle = sprintf('trackedit: Editing tracks...  ');
progbar = waitbar(0, progtitle);  % will display progress
for j = utrk
    % loop through each track
    Nframej = sum(allids==j);  % length of this track (frames)
    % Nframej = max(trtmp(5,:)) - min(trtmp(5,:));  % length of this track (frames)
    if Nframej >= minNframes
        % keep this track
        % trtmp = objs_out1(:, ismember(objs_out1(6,:),j));  % objects that are part of track j
        trtmp = objs_out1(:, allids==j);  % objects that are part of track j
        trtmp(6,:) = k;  % renumber track id
        objs_out(:,nc:(nc+size(trtmp,2))-1) = trtmp; % keep these
        k = k+1;
        nc = nc + size(trtmp,2);
        if dispopt(2)
            c = 0.7*[mod(j,2) mod(j,3)/2 1];  % for coloring
            plot([min(trtmp(5,:)) max(trtmp(5,:))], j*[1 1], '-', 'color', c);
        end
    end
    if (mod(j,100)==0)
        waitbar(j/max(utrk), progbar, strcat(progtitle, sprintf('track %d of %d', j, max(utrk))));
    end
end
if dispopt(2)
    title('Track lengths (final)')
end
close(progbar)
nc = nc-1;
fs = sprintf('Keeping %d out of %d tracks', k-1, length(utrk)); disp(fs);
objs_out = objs_out(:,1:nc);  % "re-sizing" the array

%% Display information
if dispopt(1)
    % Display information about tracks found
    fs = sprintf('%d unique tracks.', length(unique(objs_out(6,:))));
    disp(fs)
    for j=unique(objs_out(6,:))
        objs_j = objs_out(:,objs_out(6,:)==j);  % track j
        frj = objs_j(5,:);
        x = objs_j(1,:);  % x positions of this track
        y = objs_j(2,:);  % y positions of this track
        stdj = sqrt(var(x) + var(y));  % standard deviation of this track's position
        %frj = objs(5,find(objs(6,:)==j));
        fs = sprintf('   Track %d: frames %d:%d (%d frames), std. %.1f pixels',...
            j, min(frj), max(frj), length(frj), stdj);
        % Note that some frames between min and max may not be good
        disp(fs)
    end
end

